//layout bottombar toc
//layout toc       link(section)

//titlepage ------------------------------------------------------------
/*txt




{center:{bf:Extending the label commands (cont'd)}}


{center:Daniel Klein}
{center:International Centre for Higher Education Research Kassel}
{center:klein@incher.uni-kassel.de}
{center:klein.daniel.81@gmail.com}
txt*/
//endtitlepage ---------------------------------------------------------

//toctitle Extending the label commands (cont'd)


//section Motivation
// ---------------------------------------------------------------------

//slide ----------------------------------------------------------------
//title Data management

/*txt
{pstd}
Manipulating variables is convenient in Stata

{phang}
{c -} wildcard characters ({helpb varlist:*{sf:,} ~{sf:,} ?}) 
in variable names
{p_end}

{phang}
{c -} {help operators:arithmetic{sf:,} logical{sf:,} and relational eexpessions}
as well as {help functions:functions}
{p_end}

{phang}
{c -} transformation rules ({helpb recode})
{p_end}

{phang}
{c -} {help rename_group:renaming} variables is systematic
{p_end}

{pstd}
Defining and manipulating variable and value labels is not so convenient

{phang}
{c -} no wildcard characters ({bf:*{sf:,} ~{sf:,} ?}) in 
value label names
{p_end}

{phang}
{c -} eexpessions and functions do not apply to variable or value labels
{p_end}

{phang}
{c -} no transformation rules
{p_end}

{phang}
{c -} renaming value labels is not straightforward /*digr*/
{p_end}
txt*/
//endslide -------------------------------------------------------------

//digr
//title Renaming a value label
/*txt
{p 8 15 2}
Step 1 {help label:copy} the old value label using a new name
{p_end}

{p 8 15 2}
Step 2 attach the new value label to all variables that previously had 
the old value label attached
{p_end}
{p 15 15 2}
make sure to do this in all {help label_language:label languages}
{p_end}

{p 8 15 2}
Step 3 optionally, {help label:drop} the old value label
{p_end}
txt*/
//enddigr

//slide ----------------------------------------------------------------
//title Community-contributed commands to manipulate variable and value labels

/*txt
{pstd}
Many community-contibuted (collection of) commands 

{phang}
{c -} {net "describe labutil , from(http://fmwww.bc.edu/RePEc/bocode/l)":{bf:labutil} (Cox, 2000; 2013)}
{p_end}

{phang}
{c -} {net "describe dm0012 , from(http://www.stata-journal.com/software/sj5-2)":{bf:labeldup}, {bf:labelrename} (Weesie, 2005a)}
{p_end}

{phang}
{c -} {net "describe dm0013 , from(http://www.stata-journal.com/software/sj5-2)":{bf:mlanguage} (Weesie, 2005b)}
{p_end}

{phang}
{c -} {net "describe labelsof , from(http://fmwww.bc.edu/RePEc/bocode/l)":{bf:labelsof} (Jann, 2007)}
{p_end}

{phang}
{c -} {net "describe varlabdef , from(http://fmwww.bc.edu/RePEc/bocode/v)":{bf:varlabdef} (Newson, 2009)}
{p_end}

{phang}
{c -} {net "describe labutil2 , from(http://fmwww.bc.edu/RePEc/bocode/l)":{bf:labutil2} (Klein, 2011; 2013)}
{p_end}

{phang}
{c -} {net "describe vallabdef, from(http://fmwww.bc.edu/RePEc/bocode/v)":{bf:vallabdef} (Newson, 2018)}
{p_end}

{phang}
{c -} ...
{p_end}

{pstd}
Great. However, ...

{phang}
{c -} many commands are tailored to solve one specific problem
{p_end}

{phang}
{c -} need to find the (one) specific command for any specific problem
{p_end}

{phang}
{c -} need to remember (sometimes cryptic) command names and syntax 
{p_end}
txt*/
//endslide -------------------------------------------------------------

//anc ------------------------------------------------------------------
//title Specific problems and general solutions

/*txt
{pstd}
The following assumes familiarity with the {cmd:elabel} command

{pstd}
We will look at two problems and compare solutions from the 
{helpb labutil} package with {cmd:elabel}

{pstd}
{ul:Problem 1} 

{pstd}
"Define a value label for values which are base 10 logarithms
containing the antilogged values"
{p_end}
{pstd}
(from {help lablog:help lablog})
{p_end}

{phang2}
{stata lablog logs , values(1/6):. lablog logs , values(1/6)}
{p_end}

{phang2}
{stata elabel define logs2 (1/6) (= strofreal(10^#)):. elabel define logs2 (1/6) (= strofreal(10^#))}
{p_end}

{phang2}
{stata elabel list logs*:. elabel list logs*}
{p_end}

{pstd}
Compared with {cmd:elabel}, {cmd:lablog} arguably has the simpler syntax 

{pstd}
{ul:Problem 2} 

{pstd}
Define a value label for values which map minutes after midnight to hours

{phang2}
{stata labmap time , values(0(60)240) first(12) step(1) max(12) postfix(" am"):. labmap time , values(0(60)240) first(12) step(1) max(12) postfix(" am")}
{p_end}

{phang2}
{stata elabel define time2 (0(60)240) (= strofreal(cond(#, #/60, 12)) + " am"):. elabel define time2 (0(60)240) (= strofreal(cond(#, #/60, 12)) + " am")}
{p_end}

{phang2}
{stata elabel list time*:. elabel list time*}
{p_end}

{pstd}
Compared with {cmd:elabel}, does {cmd:labmap} have the simpler syntax?

{pstd}
Compared with {cmd:lablog}, the more general {cmd:labmap} has a more 
complicated syntax

{pstd}
{cmd:elabel} solves both problems with basically the same syntax 
txt*/
//endanc ---------------------------------------------------------------


//section The elabel command
// ---------------------------------------------------------------------

//slide ----------------------------------------------------------------
//title Introducing the elabel command

/*txt
{pstd}
Yet another command for manipulating variable and value labels: {helpb elabel}

{phang}
{c -} based on Stata's {helpb label} commands
{p_end}

{phang}
{c -} follows general command - subcommand structure
{p_end}

{phang}
{c -} basic syntax familiar to most Stata users 
{p_end}

{pstd}
Syntax

{phang2}
{cmd:elabel} {it:subcommand}
[ {it:elblnamelist} ]
[ {it:mappings} ]
[ {cmd:iff} {it:eexp} ]
[ {cmd:,} {it:options} ]
{p_end}
txt*/
//endslide -------------------------------------------------------------

//subsection Syntax elements

//slide ----------------------------------------------------------------
//title elabel {it:{help elabel##cmds:subcommand}} {sf:{it:elblnamelist mappings}} iff {sf:{it:eexp}} , {it:options}

/*txt
{pstd}
The subcommands are the same as those used with {cmd:label}

{synoptset 18 tabbed}{...}
{...}
{synopt:{helpb elabel_variables:elabel variable}}
Label variables{p_end}

{...}
{synopt:{helpb elabel_define:elabel define}}
Define and modify value labels{p_end}

{...}
{synopt:{helpb elabel_values:elabel values}}
Attach value label to variables{p_end}

{...}
{synopt:{helpb elabel_dir:elabel dir}}
List names of value labels{p_end}

{...}
{synopt:{helpb elabel_list:elabel list}}
List names and contents of value labels{p_end}

{...}
{synopt:{helpb elabel_copy:elabel copy}}
Copy value label{p_end}

{...}
{synopt:{helpb elabel_drop:elabel drop}}
Drop value labels{p_end}

{...}
{synopt:{helpb elabel_save:elabel save}}
Save value labels in do-file{p_end}


{pstd}
There are some additional subcommands; currently

{synoptset 18 tabbed}{...}
{...}
{synopt:{helpb elabel_compare:elabel compare}}
Compare value lables{p_end}

{...}
{synopt:{helpb elabel_keep:elabel keep}}
Keep value labels{p_end}

{...}
{synopt:{helpb elabel_load:elabel load}}
Define value lables from file{p_end}

{...}
{synopt:{helpb elabel_recode:elabel recode}}
Recode value labels{p_end}

{...}
{synopt:{helpb elabel_remove:elabel remove}}
Remove value lables from variables and memory{p_end}

{...}
{synopt:{helpb elabel_rename:elabel rename}}
Rename value labels{p_end}
txt*/
//endslide -------------------------------------------------------------

//slide ----------------------------------------------------------------
//title elabel {sf:{it:subcommand}} {it:{help elabel##elblnamelist:elblnamelist}} {sf:{it:mappings}} iff {sf:{it:eexp}} , {it:options}

/*txt
{pstd}
We start the demonstration with

{phang2}
{stata sysuse auto , clear:. sysuse auto , clear}
{p_end}
{phang2}
{stata describe:. describe}
{p_end}

{pstd}
Let us list the contents of value label {cmd:origin}

{phang2}
{stata elabel list origin:. elabel list origin}
{p_end}

{pstd}
We can abbreviate {cmd:origin} using wildcard characters

{phang2}
{stata elabel list ori~:. elabel list ori~}
{p_end}

{pstd}
We do not even need to know the value label name

{phang2}
{stata elabel list (foreign):. elabel list (foreign)}
{p_end}

{pstd}
Also, {cmd:elabel} sometimes returns additional results

{phang2}
{stata return list:. return list}
{p_end}
txt*/
//endslide -------------------------------------------------------------

//slide ----------------------------------------------------------------
//title elabel {sf:{it:subcommand elblnamelist}} {it:{help elabel##mappings:mappings}} iff {sf:{it:eexp}} , {it:options}

/*txt
{pstd}
Mappings always resemble those used with the respective {cmd:label} commands

{pstd}
We could define a new value label for {cmd:rep78} (Repair record)

{phang2}
{cmd:elabel define reprec ///}
{p_end}
{phang2}
{bind:    }{cmd:1 "very bad"}{bind:     }{cmd:///}
{p_end}
{phang2}
{bind:    }{cmd:2 "bad"}{bind:          }{cmd:///}
{p_end}
{phang2}
{bind:    }{cmd:3 "medicore"}{bind:     }{cmd:///}
{p_end}
{phang2}
{bind:    }{cmd:4 "good"}{bind:         }{cmd:///}
{p_end}
{phang2}
{bind:    }{cmd:5 "very good"}
{p_end}

{pstd}
Sometimes, there are extended mappings

{pstd}
We can group integer values and text; the above could be written as

{phang2}
{stata elabel define reprec (1/5) ("very bad" "bad" "medicore" "good" "very good"):. elabel define reprec (1/5) ("very bad" "bad" "medicore" "good" "very good")}
{p_end}
{phang2}
{stata elabel values rep78 reprec:. elabel values rep78 reprec}
{p_end}

{pstd}
We can also use eexpessions and functions; I will show you a minute
txt*/
//endslide -------------------------------------------------------------

//slide ----------------------------------------------------------------
//title elabel {sf:{it:subcommand elblnamelist mappings}} {help elabel##iffeexp:{bf:iff}} {help elabel##elabel_eexp:{it:eexp}} , {it:options}

/*txt
{pstd}
With {cmd:elabel}'s {helpb elabel##iffeexp:iff} qualifier, we refer to 
integer-to-text mappings

{pstd}
In {help elabel##elabel_eexp:{it:eexp}}, the {cmd:#} character 
represents integer values

{phang2}
{stata "version 11.2 : elabel list reprec if (# < 3)":. elabel list reprec iff (# < 3)}
{p_end}

{pstd}
while the {cmd:@} character represents text

{phang2}
{stata `"version 11.2 : elabel list reprec if strpos(@, "oo")"':. elabel list reprec iff strpos(@, "oo")}
{p_end}

{pstd}
Both characters can be combined, of course

{phang2}
{stata `"version 11.2 : elabel list reprec if (# >= 3) & !strpos(@, "oo")"':. elabel list reprec iff (# >= 3) & !strpos(@, "oo")}
{p_end}

{pstd}
We will not see any useful application in this talk; an example /*digr*/
txt*/
//endslide -------------------------------------------------------------

//digr -----------------------------------------------------------------
/*txt
{pstd}
Suppose, there is another set of value lables for a car's repair record

{phang2}
{stata elabel define repair 0 "poor" 1 "excellent" .a "not available":. elabel define repair 0 "poor" 1 "excellent" .a "not available"}
{p_end}

{pstd}
We would like to add the label for missing record to {cmd:reprec}

{phang2}
{stata "version 11.2 : elabel copy repair reprec if mi(#) , add":. elabel copy repair reprec iff mi(#) , add}
{p_end}
{phang2}
{stata elabel list reprec:. elabel list reprec}
{p_end}

{pstd}
We will not need the additional integer-to-text mapping in this talk

{phang2}
{stata elabel define reprec .a "" , modify:elabel define reprec .a "" , modify}
{p_end}
txt*/
//enddigr --------------------------------------------------------------

//slide ----------------------------------------------------------------
//title elabel {sf:{it:subcommand elblnamelist}} {it:{help elabel##mappings:mappings}} if {sf:{it:eexp}} , {it:options}

/*txt
{pstd}
I have told you that {cmd:elabel define} allows expessions 
and functions in {it:mappings}

{pstd}
Say, we want to reverse the coding for {cmd:rep78}

{phang2}
{stata tabulate rep78:. tabulate rep78}
{p_end}
{phang2}
{stata elabel list (rep78):. elabel list (rep78)}
{p_end}

{pstd}
Here is one way to reverse the coding

{phang2}
{stata replace rep78 = 6 - rep78:. replace rep78 = 6 - rep78}
{p_end}
{phang2}
{stata tabulate rep78:. tabulate rep78}
{p_end}

{pstd}
But now the labels are messed

{pstd}
Here is how we reverse the labels

{phang2}
{stata elabel define reprec (= 6 - #) (= @) , replace:. elabel define reprec (= 6 - #) (= @) , replace}
{p_end}
{phang2}
{stata tabulate rep78:. tabulate rep78}
{p_end}
txt*/
//endslide -------------------------------------------------------------

//anc ------------------------------------------------------------------
//title Variables that have no value label attached

/*txt
{pstd}
What if we refer to non-existing value labels?

{phang2}
{stata sysuse auto , clear:. sysuse auto , clear}
{p_end}
{phang2}
{stata elabel list (mpg):. elabel list (mpg)}
{p_end}
txt*/
//endanc ---------------------------------------------------------------

//anc ------------------------------------------------------------------
//title Undefined value labels

/*txt
{pstd}
{cmd:elabel dir} lists undefined value labels

{phang2}
{stata sysuse auto , clear:. sysuse auto , clear}
{p_end}
{phang2}
{stata elabel values mpg mpglbl:. elabel values mpg mpglbl}
{p_end}
{phang2}
{stata elabel dir , nomemory:. elabel dir , nomemory}
{p_end}
{phang2}
{stata return list:. return list}
{p_end}
txt*/
//endanc ---------------------------------------------------------------

//anc ------------------------------------------------------------------
//title Label multiple variables

/*txt
{pstd}
The syntax diagram for {helpb elabel_variables:elabel variable} shows 
extended mappings

{phang2}
{stata sysuse auto , clear:. sysuse auto , clear}
{p_end}
{phang2}
{stata describe rep78 foreign:. describe rep78 foreign}
{p_end}
{phang2}
{stata elabel variable rep78 "Repair Rec." foreign "Manufactured outside US":. elabel variable rep78 "Repair Rec." foreign "Manufactured outside US"}
{p_end}
{phang2}
{stata elabel variable (rep78 foreign) ("Repair Record 1978" "Car type"):. elabel variable (rep78 foreign) ("Repair Record 1978" "Car type")}
{p_end}
txt*/
//endanc ---------------------------------------------------------------

//anc ------------------------------------------------------------------
//title Reverse label codings using transformation rules

/*txt
{pstd}
Let us define a label for {cmd:rep78}

{phang2}
{stata sysuse auto , clear:. sysuse auto , clear}
{p_end}
{phang2}
{stata elabel define reprec (1/5) ("very bad" "bad" "medicore" "good" "very good"):. elabel define reprec (1/5) ("very bad" "bad" "medicore" "good" "very good")}
{p_end}
{phang2}
{stata elabel values rep78 reprec:. elabel values rep78 reprec}
{p_end}

{pstd}
We could use {helpb recode} to reverse the coding 

{phang2}
{cmd:recode rep78}{bind:            }{cmd:///}
{p_end}
{phang2}
{bind:    }{cmd:(1 = 5 "very bad")}{bind:  }{cmd:///}
{p_end}
{phang2}
{bind:    }{cmd:(2 = 4 "bad")}{bind:       }{cmd:///}
{p_end}
{phang2}
{bind:    }{cmd:(4 = 2 "good")}{bind:      }{cmd:///}
{p_end}
{phang2}
{bind:    }{cmd:(1 = 5 "very good")}{bind: }{cmd:///}
{p_end}
{phang2}
{bind:    }{cmd:, generate(rep78r)}
{p_end}

{pstd}
Defining value labels on the fly is a nice feature ...

{pstd}
But, do we really need to retype all those existing labels?

{pstd}
Here is a solution with {cmd:elabel}

{phang2}
{stata elabel recode reprec (1/5 = 5/1) , define(rep78r):. elabel recode reprec (1/5 = 5/1) , define(rep78r)}
{p_end}

{pstd}
I have mentioned that {cmd:elabel} sometimes stores useful results

{phang2}
{stata return list:. return list}
{p_end}

{pstd}
We can recode any variables accordingly

{phang2}
{stata recode rep78 `r(rules)' , generate(rep78r):. recode rep78 `r(rules)' , generate(rep78r)}
{p_end}
{phang2}
{stata elabel values rep78r rep78r:. elabel values rep78r rep78r}
{p_end}
{phang2}
{stata tabulate rep78 rep78r:. tabulate rep78 rep78r}
{p_end}
txt*/
//endanc ---------------------------------------------------------------


//anc ------------------------------------------------------------------
//title Replicating numlabel

/*txt
{pstd}
Remember our value label {cmd:reprec}?

{phang2}
{stata sysuse auto , clear:. sysuse auto , clear}
{p_end}
{phang2}
{stata elabel define reprec (1/5) ("very bad" "bad" "medicore" "good" "very good")  , replace:. elabel define reprec (1/5) ("very bad" "bad" "medicore" "good" "very good") , replace}
{p_end}
{phang2}
{stata elabel values rep78 reprec:. elabel values rep78 reprec}
{p_end}
{phang2}
{stata elabel list reprec:. elabel list reprec}
{p_end}
{phang2}
{stata tabulate rep78:. tabulate rep78}
{p_end}

{pstd}
Let us add numeric the values as a prefix to labels

{pstd}
Stata's solution is the {helpb numlabel} command

{phang2}
{stata numlabel reprec , add:. numlabel reprec , add}
{p_end}

{pstd}
Remove the prefix

{phang2}
{stata numlabel reprec , remove:. numlabel reprec , remove}
{p_end}

{pstd}
We can replicate the above with {cmd:elabel}

{phang2}
{stata elabel define reprec (= #) (= strofreal(#) + ". " + @) , modify:. elabel define reprec (= #) (= strofreal(#) + ". " + @) , modify}
{p_end}
{phang2}
{stata elabel define reprec (= #) (= subinstr(@, strofreal(#)+ ". ", "", .)) , modify:. elabel define reprec (= #) (= subinstr(@, strofreal(#)+ ". ", "", .)) , modify}
{p_end}
txt*/
//endanc ---------------------------------------------------------------


//section A close-to-real-life example
// ---------------------------------------------------------------------

//subsection Cleaning and preparing data
// ---------------------------------------------------------------------

//slide ----------------------------------------------------------------
//title Rename value labels and modify variable labels

/*txt
{pstd}
I have used the setup file for the Mikrozensus 2010 from {cmd:MISSY}
{p_end}
{pstd}
({browse "https://www.gesis.org/missy/materials/MZ/setups"})
{p_end}

{pstd}
Here is the data (for which I have actually made up the observations)

{phang2}
{stata use mz2010 , clear:. use mz2010 , clear}
{p_end}
{phang2}
{stata describe:. describe}
{p_end}

{pstd}
I do not like all uppercase names; I find them cumbersome to type

{phang2}
{stata rename * , lower:. rename * , lower}
{p_end}

{pstd}
I can do the same with value label names

{phang2}
{stata elabel rename * , lower:. elabel rename * , lower}
{p_end}

{pstd}
Perhaps, I want remove those underscores in value label names /*digr*/

{pstd}
Let us also remove those {cmd:F{it:#}} prefixes from variable labels

{phang2}
{stata elabel variable (*) (= regexr(@, "^F[0-9]*[ ]", "")):. elabel variable (*) (= regexr(@, "^F[0-9]*[ ]", ""))}
{p_end}
txt*/
//endslide -------------------------------------------------------------

//digr
/*txt

{phang2}
{stata elabel rename (*_*) (**) , dryrun:. elabel rename (*_*) (**) , dryrun}
{p_end}
txt*/
//enddigr

//slide ----------------------------------------------------------------
//title Identify and eliminate identical value labels

/*txt
{pstd}
Based on variable labels, some of the variables seem to contain similar information

{pstd}
I wonder why the all have their own value label attached

{pstd}
Let us look at the those value labels

{phang2}
{stata elabel list (ef670-ef673):. elabel list (ef670-ef673)}
{p_end}

{pstd}
As expected, those labels look pretty similar

{pstd}
{helpb labeldup} (not part of {helpb elabel}) identifies and removes 
duplicate labels

{phang2}
{stata labeldup , select:. labeldup , select}
{p_end}
{phang2}
{stata describe:. describe}
{p_end}

{pstd}
How is {cmd:ef673_vl} different from {cmd:ef670_vl}

{phang2}
{stata elabel compare ef673_vl ef670_vl:. elabel compare ef673_vl ef670_vl}
{p_end}

{pstd}
Both labels are identical if we exclude value 5 /*digr*/

{pstd}
We should attach {cmd:ef673_vl} to {cmd:ef670}, {cmd:ef671}, and {cmd:ef672}

{phang2}
{stata elabel values ef670-ef672 (ef673):. elabel values ef670-ef672 (ef673)}
{p_end}

{pstd}
We do no longer need value label {cmd:ef670_vl}

{phang2}
{stata elabel drop ef670_vl:. elabel drop ef670_vl}
{p_end}
txt*/
//endslide -------------------------------------------------------------

//digr -----------------------------------------------------------------
//title Compare parts of value labels

/*txt
{phang2}
{stata "version 11.2 : elabel compare ef673_vl ef670_vl if (# < 5)":. elabel compare ef673_vl ef670_vl iff (# < 5)}
{p_end}
txt*/
//enddigr --------------------------------------------------------------

//slide ----------------------------------------------------------------
//title Change numeric values to extended missing values

/*txt
{pstd}
Next, I would like to define missing values

{pstd}
Consider

{phang2}
{stata elabel list:. elabel list}
{p_end}

{pstd}
It seems as if consecutive negative numbers, -1, -2, ..., -5 indicate 
missing values

{pstd}
I would like to have consecutive extended missing values, .a, .b, ..., .e 

{pstd}
Of course, I would like to keep the value labels

{pstd}
Here is how we do this for value label {cmd:ef1109_vl}

{phang2}
{stata elabel recode ef1109_vl (-1/-5 = .a/.e):. elabel recode ef1109_vl (-1/-5 = .a/.e)}
{p_end}
{phang2}
{stata return list:. return list}
{p_end}
{phang2}
{stata local rules `r(rules)':. local rules `r(rules)'}
{p_end}

{pstd}
Value label {cmd:ef1109_vl} is attached to more than one variable

{pstd}
We need to {helpb recode} all variables accordingly

{phang2}
{stata elabel list ef1109_vl , varlist:. elabel list ef1109_vl , varlist}
{p_end}
{phang2}
{stata return list:. return list}
{p_end}
{phang2}
{stata local varlist `r(varlist)':. local varlist `r(varlist)'}
{p_end}
{phang2}
{stata recode `varlist' `rules':. recode `varlist' `rules'}
{p_end}
txt*/
//endslide -------------------------------------------------------------


//section Adding new commands to elabel
// ---------------------------------------------------------------------

//subsection How to?

//slide ----------------------------------------------------------------
//title Adding new commands  to elabel

/*txt
{pstd}
There are two ways to add commands to 
{helpb elabel_programming##addcmd:elabel}

{phang}
{c -} write a program, {cmd:elabel_cmd_{it:newcmd}} (in a minute)
{p_end}

{phang}
{c -} write a program, {cmd:elabel_fcn_{it:newfcn}} 
(see {help elabel_functions##addfcn:elabel (pseudo-)functions})
{p_end}

{phang}
we can also use {helpb elabel_programming:elabel{sf:'s programming tools}} 
in any program (or do-file)
{p_end}
txt*/
//endslide -------------------------------------------------------------

//subsection Detach value labels from variables

//slide ----------------------------------------------------------------
//title Detach value labels from variables

/*txt
{pstd}
{cmd:label values} attach and detaches value labels to a 
list of variables

{pstd}
Suppose, we wanted to detach a list of value labels from all variables

{pstd}
We want a command with the following syntax

{phang2}
{cmd:elabel detach} {it:elblnamelist}
{p_end}

{pstd}
Here is how we implement this
txt*/
//ex
program elabel_cmd_detach
    version 15
    elabel parse elblnamelist : `0'
    foreach lbl of local lblnamelist {
        quietly elabel list `lbl' , varlist
        if ("`r(varlist)'" != "") elabel values `r(varlist)' .
    }
end
//endex
/*txt
{pstd}
{helpb elabel_parse:elabel parse} resembles a rudimentary version 
of {helpb syntax}

{pstd}
Let us try our command

{phang2}
{stata describe:. describe}
{p_end}
{phang2}
{stata elabel detach *673*:. elabel detach *673*}
{p_end} 
txt*/
//endslide -------------------------------------------------------------

//anc ------------------------------------------------------------------
//title Delete integer-to-text mappings from value labels

/*txt
{pstd}
Syntax

{phang2}
{cmd:elabel delete} {help elabel##elblnamelist:{it:elblnamelist}}
{ {help numlist:{it:numlist}} | {help elabel##iffeexp:{bf:iff {sf:{it:eexp}}}} }
[ , {opt not} ]
{p_end}
txt*/
//ex
*! version 1.0.0 24may2019 daniel klein
program elabel_cmd_delete
    version 11.2
    
    elabel parse elblnamelist [ mappings ] [ iff ] [ , NOT ] : `0'
    
    if mi(`"`iff'`mappings'"') {
        display as err "iff eexp required"
        exit 100
    }
    else if ((`"`iff'"' != "") & (`"`mappings'"' != "")) {
        display as err "iff not allowed"
        exit 101
    }
    
    if ("`not'" == "not") local not !
    
    if (`"`mappings'"' != "") {
        elabel protectr
        elabel numlist `"`mappings'"'
        local numlist `r(numlist)'
        local numlist : subinstr local numlist " " ", " , all
        local iffeexp inlist(#, `numlist')
    }
    else gettoken ifword iffeexp : iff
    
    local iffeexp `not'(`iffeexp')
    
    elabel define `lblnamelist' (= #) (= cond(`iffeexp', "", @)) , modify
end
exit
//endex
//endanc ---------------------------------------------------------------

//anc ------------------------------------------------------------------
//title A wrapper for labeldup

/*txt
{pstd}
Syntax

{phang2}
{cmd:elabel dup} {help elabel##elblnamelist:{it:elblnamelist1}}
[ {help elabel##iffeexp:{bf:iff {sf:{it:eexp}}}} ]
[ , {opt s:elect} 
{cmdab:n:ames(}{help elabel##elblnamelist:{it:elblnamelist2}}{cmd:)}
{opt nod:rop} ]
{p_end}
txt*/
//ex
*! version 1.0.0 daniel klein 24may2019
program elabel_cmd_dup
    version 11.2
    
    elabel parse [ elblnamelist ] [ iff ]   ///
        [ , Select Names(string asis) * ] : `0'
    
    if (`"`names'"' != "") elabel unab labellist2 : `names' , elblnamelist
    
    preserve
    
    if (`"`iff'"' != "") {
        tempfile tmp
        quietly label save using "`tmp'"
        label drop _all
        elabel load `iff' using "`tmp'" , as(do)
    }
    
    labeldup `lblnamelist' , `select' names(`labellist2') `options'
    
    restore , not
    
    if (mi(`"`iff'"')) exit
    
    if ("`select'" == "select") elabel unab toload : *
    
    elabel load `toload' using "`tmp'" , modify as(do)
end
exit
//endex
//endanc ---------------------------------------------------------------
